﻿'restricted rights notice below appear in all supporting 
'documentation.

'AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
'AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
'MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
'DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
'UNINTERRUPTED OR ERROR FREE.

'Use, duplication, or disclosure by the U.S. Government is subject to 
'restrictions set forth in FAR 52.227-19 (Commercial Computer
'Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
'(Rights in Technical Data and Computer Software), as applicable.

'This is required to use "dynamic" feature in VB.Net
Option Explicit On
Option Strict Off
Option Infer On

Imports System
Imports System.Linq

Imports Autodesk.AutoCAD.Runtime
Imports Autodesk.AutoCAD.ApplicationServices
Imports Autodesk.AutoCAD.DatabaseServices
Imports Autodesk.AutoCAD.Geometry
Imports Autodesk.AutoCAD.EditorInput

<Assembly: CommandClass(GetType(DynamicNetTestVB.MyCommands))> 

'///////////////////////////////////////////////////////////////////////////////////////////////////////
' DynamicNetTestVB is a sample project that illustrates the new dynamic 
' features of the AutoCAD 2013 .Net API
'
' Products: AutoCAD (and verticals) 2013 and higher
'
' Written by Philippe Leefsma, October 2011 - Developer Technical Services
'
'///////////////////////////////////////////////////////////////////////////////////////////////////////
Namespace DynamicNetTestVB

    Public Class MyCommands

        'Description: A VB.NET port of the C# dynamic example with Linq queries

        <CommandMethod("DynamicTestGroup", "_DynamiDbcTestVb", "DynamicDbTestVb", CommandFlags.Modal)> _
        Public Sub DynamicDbTestVb()

            Dim doc As Document = Application.DocumentManager.MdiActiveDocument

            'bootstrap dynamic typing. once db is 'dynamic' we use all intellisense and all expressions will be dynamic too
            Dim db As Object = HostApplicationServices.WorkingDatabase

            'do some tests on layers
            Dim layers As Object = db.LayerTableId

            For i As Integer = 0 To 100

                Dim newrec = layers.Add(New LayerTableRecord())
                newrec.Name = "Layer " + i.ToString()

                If (i Mod 4 = 0) Then
                    newrec.IsFrozen = True
                ElseIf (i Mod 5 = 0) Then
                    newrec.IsOff = True
                End If

            Next

            'store layer5 so that we don't have to look it up each time through the loop
            Dim layer5 As Object = layers.get_Item("Layer 5")

            'do some tests on blocks
            Dim modelspace = db.BlockTableId.get_Item(BlockTableRecord.ModelSpace)

            For i As Integer = 0 To 100

                Dim newent

                If i Mod 3 = 0 Then
                    newent = modelspace.AppendEntity(New Line(New Point3d(i, 0, 0), New Point3d(0, i, 0)))
                Else
                    newent = modelspace.AppendEntity(New Circle(New Point3d(i, i, 0), Vector3d.ZAxis, i))
                End If

                If (i Mod 5 = 0) Then
                    newent.LayerId = layer5
                End If
            Next

            'now some dictionary stuff
            Dim nod As Object = db.NamedObjectsDictionaryId
            Dim newdict = nod.SetAt("DynaTestDict", New DBDictionary())
            Dim record = newdict.SetAt("DynaTestRecord", New Xrecord())

            Dim data As String = "This is a dynamic test"

            record.Data = New ResultBuffer(New TypedValue(DxfCode.Text, data))

            'not working...
            'Dim dict As Object = nod.get_Item("DynaTestDict")
            'Dim xr As Object = dict.get_Item("DynaTestRecord")
            'doc.Editor.WriteMessage(vbCrLf + " - " + xr.Data.ToString())

        End Sub

    End Class

End Namespace